/*
 * Copyright 2016 - 2022 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#pragma once

#include <ox/std/array.hpp>
#include <ox/std/types.hpp>
#include <ox/model/def.hpp>

#include <nostalgia/geo/point.hpp>
#include <nostalgia/geo/size.hpp>

#include "color.hpp"
#include "context.hpp"
#include "ptidxconv.hpp"
#include "tilesheet.hpp"

namespace nostalgia::core {

extern ox::Array<char, 128> charMap;

class Drawer {
	public:
		virtual ~Drawer() = default;

		virtual void draw(Context*) noexcept = 0;
};

enum class TileSheetSpace {
	Background,
	Sprite
};

struct Sprite {
	unsigned idx = 0;
	int x = 0;
	int y = 0;
	unsigned tileIdx = 0;
	unsigned spriteShape = 0;
	unsigned spriteSize = 0;
	unsigned flipX = 0;
};

oxModelBegin(Sprite)
	oxModelField(idx)
	oxModelField(x)
	oxModelField(y)
	oxModelField(tileIdx)
	oxModelField(spriteShape)
	oxModelField(spriteSize)
	oxModelField(flipX)
oxModelEnd()

ox::Error initGfx(Context *ctx) noexcept;

void addCustomDrawer(Context *ctx, Drawer *cd) noexcept;

void removeCustomDrawer(Context *ctx, Drawer *cd) noexcept;

void setWindowTitle(Context *ctx, ox::CRStringView title) noexcept;

void focusWindow(Context *ctx) noexcept;

[[nodiscard]]
int getScreenWidth(Context *ctx) noexcept;

[[nodiscard]]
int getScreenHeight(Context *ctx) noexcept;

[[nodiscard]]
geo::Size getScreenSize(Context *ctx) noexcept;

[[nodiscard]]
uint8_t bgStatus(Context *ctx) noexcept;

void setBgStatus(Context *ctx, uint32_t status) noexcept;

bool bgStatus(Context *ctx, unsigned bg) noexcept;

void setBgStatus(Context *ctx, unsigned bg, bool status) noexcept;

ox::Error initConsole(Context *ctx) noexcept;

void setBgCbb(Context *ctx, unsigned bgIdx, unsigned cbb) noexcept;

/**
 * @param section describes which section of the selected TileSheetSpace to use (e.g. MEM_PALLETE_BG[section])
 */
ox::Error loadBgTileSheet(Context *ctx, unsigned cbb, const ox::FileAddress &tilesheetAddr,
                          const ox::FileAddress &paletteAddr = nullptr) noexcept;

ox::Error loadSpriteTileSheet(Context *ctx,
                              const ox::FileAddress &tilesheetAddr,
                              const ox::FileAddress &paletteAddr) noexcept;

void puts(Context *ctx, int column, int row, ox::CRStringView str) noexcept;

void setTile(Context *ctx, unsigned bgIdx, int column, int row, uint8_t tile) noexcept;

void clearTileLayer(Context *ctx, unsigned bgIdx) noexcept;

void hideSprite(Context *ctx, unsigned) noexcept;

void setSprite(Context *ctx, unsigned idx, int x, int y, unsigned tileIdx,
               unsigned spriteShape = 0, unsigned spriteSize = 0, unsigned flipX = 0) noexcept;

void setSprite(Context *ctx, const Sprite &s) noexcept;

}
